# -*- coding: utf-8 -*-
"default plugin for setting: set it in a simple dictionary"
# Copyright (C) 2013 Team tiramisu (see AUTHORS for all contributors)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# ____________________________________________________________
from tiramisu.plugins.sqlite3.cache import Cache


class Settings(Cache):
    __slots__ = tuple()

    def __init__(self, config_id):
        settings_table = 'CREATE TABLE IF NOT EXISTS property(path text primary key, properties text)'
        permissives_table = 'CREATE TABLE IF NOT EXISTS permissive(path text primary key, permissives text)'
        # should init cache too
        super(Settings, self).__init__(config_id, 'property')
        self._cursor.execute(settings_table)
        self._cursor.execute(permissives_table)
        self._conn.commit()

    # propertives
    def setproperties(self, path, properties):
        self._cursor.execute("DELETE FROM property WHERE path = ?", (path,))
        self._cursor.execute("INSERT INTO property(path, properties) VALUES (?, ?)",
                             (path, self._sqlite_encode(properties)))
        self._conn.commit()

    def getproperties(self, path, default_properties):
        self._cursor.execute("SELECT properties FROM property WHERE path = ?", (path,))
        value = self._cursor.fetchone()
        if value is None:
            return set(default_properties)
        else:
            return set(self._sqlite_decode(value[0]))

    def hasproperties(self, path):
        return self._cursor.execute("SELECT properties FROM property WHERE path = ?", (path,)) is not None

    def reset_all_propertives(self):
        self._cursor.execute("DELETE FROM property")
        self._conn.commit()

    def reset_properties(self, path):
        self._cursor.execute("DELETE FROM property WHERE path = ?", (path,))
        self._conn.commit()

    def get_properties(self, context):
        """return all properties in a dictionary
        """
        self._cursor.execute("SELECT * FROM property")
        ret = {}
        for path, properties in self._cursor.fetchall():
            if path == '_none':
                opt = None
            else:
                opt = context.cfgimpl_get_description().impl_get_opt_by_path(path)
            properties = self._sqlite_decode(properties)
            ret[opt] = properties
        return ret

    # permissive
    def setpermissive(self, path, permissive):
        self._cursor.execute("DELETE FROM permissive WHERE path = ?", (path,))
        self._cursor.execute("INSERT INTO permissive(path, permissives) VALUES (?, ?)",
                             (path, self._sqlite_encode(permissive)))
        self._conn.commit()

    def getpermissive(self, path='_none'):
        self._cursor.execute("SELECT permissives FROM permissive WHERE path = ?", (path,))
        permissives = self._cursor.fetchone()
        if permissives is None:
            return frozenset()
        else:
            return frozenset(self._sqlite_decode(permissives[0]))
